@@ -1,13 +1,13 @@
 #ifndef SDPARM_H
 #define SDPARM_H
 
-/* sdparm is a utility program for the Linux OS SCSI subsystem.
- *
- * This utility fetches various parameters associated with a given
- * SCSI disk (or a disk that uses, or translates the SCSI command
- * set). In some cases these parameters can be changed.
+/*
+ * sdparm is a utility program for getting and setting parameters on devices
+ * that use one of the SCSI command sets. In some cases commands can be sent
+ * to the device (e.g. eject removable media).
  */
 
+#include <stdint.h>
 
 #define DEF_MODE_RESP_LEN 252
 #define DEF_INQ_RESP_LEN 252
@@ -27,6 +27,7 @@
 #define CONTROL_MP 0xa
 #define POWER_OLD_MP 0xd
 /* #define CD_DEV_PARAMS 0xd */
+#define ADC_MP 0xe
 #define DATA_COMPR_MP 0xf
 #define DEV_CONF_MP 0x10
 #define XOR_MP 0x10
@@ -45,7 +46,6 @@
 #define ALL_MPAGES 0x3f
 
 /* Mode subpage numbers */
-#define MSP_CONTROL_EXT 1       /* unused */
 #define MSP_SPC_CE 1            /* control extension */
 #define MSP_SPI_MC 1
 #define MSP_SPI_STC 2
@@ -53,11 +53,19 @@
 #define MSP_SPI_RTC 4
 #define MSP_SAS_PCD 1
 #define MSP_SAS_SPC 2
-#define MSP_SAS2_PHY 3
+#define MSP_SAS_E_PHY 3
 #define MSP_BACK_CTL 1
 #define MSP_SAT_PATA 0xf1       /* SAT PATA Control */
+#define MSP_SAT_POWER 0xf1      /* SAT ATA Power condition */
 #define MSP_DEV_CONF_EXT 1      /* device conf extension (ssc) */
 #define MSP_EXT_DEV_CAP 0x41    /* extended device capabilities (smc) */
+#define MSP_ADC_TGT_DEV 0x1
+#define MSP_ADC_DT_DPP 0x2
+#define MSP_ADC_LU 0x3
+#define MSP_ADC_TD_SN 0x4
+#define MSP_SBC_LB_PROV 0x2
+#define MSP_SSC_CDP 0xf0
+#define MSP_SBC_APP_TAG 0xf0
 
 #define MODE_DATA_OVERHEAD 128
 #define EBUFF_SZ 256
@@ -77,6 +85,8 @@
 #define VPD_MODE_PG_POLICY 0x87
 #define VPD_SCSI_PORTS 0x88
 #define VPD_ATA_INFO 0x89
+#define VPD_POWER_CONDITION 0x8a
+#define VPD_DEVICE_CONSTITUENTS 0x8b
 #define VPD_PROTO_LU 0x90
 #define VPD_PROTO_PORT 0x91
 #define VPD_BLOCK_LIMITS 0xb0   /* SBC-3 */
@@ -86,6 +96,10 @@
 #define VPD_MAN_ASS_SN 0xb1     /* SSC-3, ADC-2 */
 #define VPD_SECURITY_TOKEN 0xb1 /* OSD */
 #define VPD_TA_SUPPORTED 0xb2   /* SSC-3 */
+#define VPD_LB_PROVISIONING 0xb2   /* SBC-3 */
+#define VPD_REFERRALS 0xb3
+#define VPD_AUTOMATION_DEV_SN 0xb3   /* SSC-3 */
+#define VPD_DTDE_ADDRESS 0xb4   /* SSC-4 */
 
 #define VPD_ASSOC_LU 0
 #define VPD_ASSOC_TPORT 1
@@ -120,6 +134,8 @@
 #define CMD_SENSE 7
 #define CMD_SYNC 8
 #define CMD_CAPACITY 9
+#define CMD_SPEED 10
+#define CMD_PROFILE 11
 
 
 struct sdparm_opt_coll {
@@ -135,6 +151,7 @@ struct sdparm_opt_coll {
     int mode_6;
     int num_desc;
     int quiet;
+    int read_only;
     int save;
     int transport;
     int vendor;
@@ -142,15 +159,17 @@ struct sdparm_opt_coll {
 };
 
 struct sdparm_mode_descriptor_t {
-    int num_descs_off;
-    int num_descs_bytes;
-    int num_descs_inc;    /* number to add to num_descs derived from */
-                          /* first 2 entries */
+    int num_descs_off;    /* byte offset of start of num_descriptors */
+    int num_descs_bytes;  /* number of bytes in num_descriptors field */
+    int num_descs_inc;    /* number to add to num_descriptors */
+                          /* if negative then value in num_descriptors */
+                          /* is byte count, so divide by desc_len */
     int first_desc_off;
     int desc_len;         /* -1 for unknown otherwise fixed per desc */
     int desc_len_off;     /* if (-1 == desc_len) then this is offset */
     int desc_len_bytes;   /* ... after start of descriptor */
-    /* Hence: <desc_len> = *(d_len_off + base) + d_len_off + d_len_bytes */
+    /* Hence: <desc_len> = deref(base + d_len_off, d_len_bytes) + */
+    /*                     d_len_off + d_len_bytes */
     const char * name;
 };
 
@@ -203,8 +222,8 @@ struct sdparm_mode_page_item {
 
 struct sdparm_mode_page_it_val {
     struct sdparm_mode_page_item mpi;
-    long long val;
-    long long orig_val;
+    int64_t val;
+    int64_t orig_val;
     int descriptor_num;
 };
 
@@ -225,9 +244,16 @@ struct sdparm_vendor_pair {
     struct sdparm_mode_page_item * mitem;
 };
 
-struct sdparm_command {
+struct sdparm_command_t {
     int cmd_num;
     const char * name;
+    const char * min_abbrev;
+    const char * extra_arg;
+};
+
+struct sdparm_val_desc_t {
+        int val;
+        const char * desc;
 };
 
 extern struct sdparm_mode_page_t sdparm_gen_mode_pg[];
@@ -238,7 +264,8 @@ extern struct sdparm_vendor_name_t sdparm_vendor_id[];
 extern struct sdparm_vendor_pair sdparm_vendor_mp[];
 extern int sdparm_vendor_mp_len;
 extern struct sdparm_mode_page_item sdparm_mitem_arr[];
-extern struct sdparm_command sdparm_command_arr[];
+extern struct sdparm_command_t sdparm_command_arr[];
+extern struct sdparm_val_desc_t sdparm_profile_arr[];
 
 extern const char * sdparm_pdt_doc_strs[];
 extern const char * sdparm_transport_proto_arr[];
@@ -278,22 +305,22 @@ extern const struct sdparm_vendor_pair * sdp_get_vendor_pair(int vendor_num);
 extern const struct sdparm_mode_page_item * sdp_find_mitem_by_acron(
                 const char * ap, int * from, int transp_proto,
                 int vendor_num);
-extern unsigned long long sdp_get_big_endian(const unsigned char * from,
-                int start_bit, int num_bits);
-extern void sdp_set_big_endian(unsigned long long val, unsigned char * to,
-                int start_bit, int num_bits);
-extern unsigned long long sdp_mp_get_value(
-                const struct sdparm_mode_page_item *mpi,
-                const unsigned char * mp);
-extern unsigned long long sdp_mp_get_value_check(
+extern uint64_t sdp_get_big_endian(const unsigned char * from, int start_bit,
+                                   int num_bits);
+extern void sdp_set_big_endian(uint64_t val, unsigned char * to, int start_bit,
+                               int num_bits);
+extern uint64_t sdp_mp_get_value(const struct sdparm_mode_page_item *mpi,
+                                 const unsigned char * mp);
+extern uint64_t sdp_mp_get_value_check(
                 const struct sdparm_mode_page_item *mpi,
                 const unsigned char * mp, int * all_set);
-extern void sdp_mp_set_value(unsigned long long val,
+extern void sdp_mp_set_value(uint64_t val,
                 const struct sdparm_mode_page_item *mpi, unsigned char * mp);
 extern char * sdp_get_ansi_version_str(int version, int buff_len,
                 char * buff);
 extern char * sdp_get_pdt_doc_str(int version, int buff_len,
                 char * buff);
+extern int sdp_strcase_eq(const char * s1p, const char * s2p);
 
 /*
  * Declarations for functions found in sdparm_vpd.c
@@ -307,19 +334,19 @@ extern int sdp_process_vpd_page(int sg_fd, int pn, int spn,
  * Declarations for functions found in sdparm_cmd.c
  */
 
-extern const struct sdparm_command * sdp_build_cmd(const char * cmd_str,
-                int * rwp);
+extern const struct sdparm_command_t * sdp_build_cmd(const char * cmd_str,
+                int * rwp, int * argp);
 extern void sdp_enumerate_commands();
-extern int sdp_process_cmd(int sg_fd, const struct sdparm_command * scmdp,
-                int pdt, const struct sdparm_opt_coll * opts);
+extern int sdp_process_cmd(int sg_fd, const struct sdparm_command_t * scmdp,
+                int cmd_arg, int pdt, const struct sdparm_opt_coll * opts);
 
 /*
  * Declarations for functions that are port dependant
  */
 
-#ifdef SDPARM_WIN32
+#ifdef SG_LIB_WIN32
 
-extern int sg_do_wscan(char letter, int verbose);
+extern int sg_do_wscan(char letter, int do_scan, int verbose);
 
 #endif
 
